@@ -39,6 +39,8 @@ class ProfileRequestForm(forms.ModelForm):
         keys = [i for i in self.fields if i != "confirm_password"]
         keys.insert(keys.index("password") + 1, "confirm_password")
         self.fields = OrderedDict((i, self.fields[i]) for i in keys)
+        self.fields['password'].widget = \
+            forms.PasswordInput(attrs={'size':'50'})
 
     def clean_username(self):
         username = self.cleaned_data["username"]
@@ -265,10 +267,14 @@ class UpdateUserForm(forms.ModelForm):
         fields = ("first_name", "last_name", "email", "is_active", "is_staff",
                   "is_superuser")
 
+    def __init__(self, *args, **kwargs):
+        self.profile = kwargs.pop('profile')
+        super(UpdateUserForm, self).__init__(*args, **kwargs)
+
     def clean_is_superuser(self):
         is_superuser = self.cleaned_data["is_superuser"]
         if User.objects.filter(is_superuser=True).count() <= 1 and \
-          not is_superuser:
+          self.instance == self.profile and not is_superuser:
             raise forms.ValidationError(MESSAGES['LAST_SUPERADMIN'])
         return is_superuser
 
@@ -284,8 +290,9 @@ class FullProfileForm(forms.ModelForm):
     ''' Form to modify an existing user and profile. '''
     class Meta:
         model = UserProfile
-        fields = ("email_visible", "phone_number", "phone_visible", "status",
-                  "former_houses")
+        fields = ("email_visible", "phone_number", "phone_visible",
+                  "status", "former_houses",
+                  "email_announcement_notifications")
 
 class ModifyProfileRequestForm(forms.ModelForm):
     ''' Form to modify a profile request. '''
@@ -394,7 +401,8 @@ class UpdateProfileForm(forms.ModelForm):
     ''' Form for a user to update own profile. '''
     class Meta:
         model = UserProfile
-        fields = ("email_visible", "phone_number", "phone_visible", "former_houses")
+        fields = ("email_visible", "phone_number", "phone_visible",
+                  "former_houses", "email_announcement_notifications")
 
     enter_password = forms.CharField(
         required=False,
@@ -421,7 +429,7 @@ class UpdateEmailForm(forms.ModelForm):
     def clean_email(self):
         email = self.cleaned_data["email"]
         if User.objects.filter(email=email).count() > 0 and \
-          User.objects.get(email=email) != self.instance.user:
+          User.objects.get(email=email) != self.instance:
             raise forms.ValidationError(MESSAGES['EMAIL_TAKEN'])
         return email
 
@@ -429,16 +437,18 @@ class LoginForm(forms.Form):
     ''' Form to login. '''
     username_or_email = forms.CharField(
         max_length=100,
+        required=True,
         widget=forms.TextInput(attrs={'size':'50'}),
         )
     password = forms.CharField(
         max_length=100,
+        required=True,
         widget=forms.PasswordInput(attrs={'size':'50'}),
         )
 
     def clean(self):
-        username_or_email = self.cleaned_data['username_or_email']
-        password = self.cleaned_data['password']
+        username_or_email = self.cleaned_data.get('username_or_email', "")
+        password = self.cleaned_data.get('password', "")
         if username_or_email == ANONYMOUS_USERNAME:
             raise forms.ValidationError(MESSAGES['ANONYMOUS_DENIED'])
         try:
