@@ -10,6 +10,7 @@ Views for base application.
 from datetime import timedelta
 import time
 from smtplib import SMTPException
+import json
 
 from django.conf import settings
 from django.contrib import messages
@@ -21,11 +22,13 @@ from django.contrib.auth.views import password_reset, password_reset_confirm
 from django.core.urlresolvers import reverse
 from django.core.mail import send_mail
 from django.db.models import Q
-from django.http import HttpResponseRedirect
+from django.http import HttpResponseRedirect, HttpResponse, Http404
 from django.shortcuts import render_to_response, render, get_object_or_404
 from django.template import RequestContext
 from django.utils.timezone import now
 
+from wiki.models import Revision
+
 from utils.funcs import form_add_error
 from utils.variables import ANONYMOUS_USERNAME, MESSAGES, APPROVAL_SUBJECT, \
     APPROVAL_EMAIL, DELETION_SUBJECT, DELETION_EMAIL, SUBMISSION_SUBJECT, \
@@ -41,8 +44,10 @@ from threads.models import Thread, Message
 from threads.forms import ThreadForm
 from managers.models import RequestType, Manager, Request, Response, Announcement
 from managers.forms import AnnouncementForm, ManagerResponseForm, VoteForm, PinForm
+from managers.ajax import build_ajax_votes
 from events.models import Event
 from events.forms import RsvpForm
+from events.ajax import build_ajax_rsvps
 from rooms.models import Room, PreviousResident
 from workshift.models import Semester, WorkshiftProfile, ShiftLogEntry, \
     WorkshiftInstance
@@ -68,7 +73,7 @@ def add_context(request):
         for request_type in RequestType.objects.filter(enabled=True):
             requests = Request.objects.filter(request_type=request_type, status=Request.OPEN)
             if not request_type.managers.filter(incumbent__user=request.user):
-                requests.exclude(
+                requests = requests.exclude(
                     ~Q(owner__user=request.user), private=True,
                     )
             request_types.append((request_type, requests.count()))
@@ -81,12 +86,38 @@ def add_context(request):
         'NUM_OF_PROFILE_REQUESTS': ProfileRequest.objects.all().count(),
         'ANONYMOUS_SESSION': ANONYMOUS_SESSION,
         'PRESIDENT': PRESIDENT,
+        "WIKI_ENABLED": "farnswiki" in settings.INSTALLED_APPS,
         }
 
 def landing_view(request):
     ''' The external landing.'''
+    revision = None
+    can_edit = False
+    edit_url = None
+
+    if "farnswiki" in settings.INSTALLED_APPS:
+        from wiki.models import Page
+        from wiki.hooks import hookset
+        binder = settings.WIKI_BINDERS[0]
+        wiki = binder.lookup()
+        try:
+            if wiki:
+                page = wiki.pages.get(slug="landing")
+            else:
+                page = Page.objects.get(slug="landing")
+        except (Page.DoesNotExist, Page.MultipleObjectsReturned):
+            can_edit = hookset.can_create_page(wiki, request.user, slug="landing")
+            edit_url = binder.page_url(wiki, "landing")
+        else:
+            revision = page.revisions.latest()
+            can_edit = hookset.can_edit_page(page, request.user)
+            edit_url = page.get_edit_url()
+
     return render_to_response('external.html', {
-        'page_name': "Landing",
+        "page_name": "Landing",
+        "revision": revision,
+        "can_edit": can_edit,
+        "edit_url": edit_url,
         }, context_instance=RequestContext(request))
 
 @profile_required(redirect_no_user='external', redirect_profile=red_ext)
@@ -121,7 +152,7 @@ def homepage_view(request, message=None):
                     initial={'action': Response.NONE},
                     profile=userProfile,
                     request=req,
-                    prefix="response",
+                    prefix="{}-response".format(req.pk),
                     )
                 vote_form = VoteForm(
                     request.POST if "vote-{0}".format(req.pk) in request.POST else None,
@@ -173,14 +204,17 @@ def homepage_view(request, message=None):
                 return HttpResponseRedirect(reverse('homepage'))
         announcements_dict.append((a, pin_form))
 
-    announcement_form = AnnouncementForm(
-        request.POST if "post_announcement" in request.POST else None,
-        profile=userProfile,
-        prefix="announce",
-        )
+    if Manager.objects.filter(incumbent=userProfile, active=True).count():
+        announcement_form = AnnouncementForm(
+            request.POST if "post_announcement" in request.POST else None,
+            profile=userProfile,
+            prefix="announce",
+            )
+    else:
+        announcement_form = None
 
-    if announcement_form.is_valid():
-        announcement_form.save()
+    if announcement_form and announcement_form.is_valid():
+        announcement_form.save(request)
         return HttpResponseRedirect(reverse('homepage'))
 
     ### Events
@@ -206,7 +240,7 @@ def homepage_view(request, message=None):
 
         if rsvp_form.is_valid():
             rsvpd = rsvp_form.save()
-            if rsvpd:
+            if not rsvpd:
                 message = MESSAGES['RSVP_REMOVE'].format(event=event.title)
                 messages.add_message(request, messages.SUCCESS, message)
             else:
@@ -568,6 +602,7 @@ def custom_modify_user_view(request, targetUsername):
     update_user_form = UpdateUserForm(
         request.POST if "update_user_profile" in request.POST else None,
         instance=targetUser,
+        profile = UserProfile.objects.get(user=request.user),
         prefix="user",
         )
     update_profile_form = FullProfileForm(
@@ -611,7 +646,7 @@ def custom_modify_user_view(request, targetUsername):
             )
         return HttpResponseRedirect(reverse("custom_manage_users"))
 
-    return render_to_response('custom_modify_user.html', {
+    template_dict = {
         'targetUser': targetUser,
         'targetProfile': targetProfile,
         'page_name': page_name,
@@ -619,13 +654,31 @@ def custom_modify_user_view(request, targetUsername):
         'update_profile_form': update_profile_form,
         'change_user_password_form': change_user_password_form,
         'delete_user_form': delete_user_form,
-        'thread_count': Thread.objects.filter(owner=targetProfile).count(),
-        'message_count': Message.objects.filter(owner=targetProfile).count(),
-        'request_count': Request.objects.filter(owner=targetProfile).count(),
-        'response_count': Response.objects.filter(owner=targetProfile).count(),
-        'announcement_count': Announcement.objects.filter(incumbent=targetProfile).count(),
-        'event_count': Event.objects.filter(owner=targetProfile).count(),
-        }, context_instance=RequestContext(request))
+        }
+
+    if "wiki" in settings.INSTALLED_APPS:
+        from wiki.models import Revision
+        template_dict["revision_count"] = \
+          Revision.objects.filter(created_by=targetUser).count()
+
+    template_dict['thread_count'] = \
+      Thread.objects.filter(owner=targetProfile).count()
+    template_dict['message_count'] = \
+      Message.objects.filter(owner=targetProfile).count()
+    template_dict['request_count'] = \
+      Request.objects.filter(owner=targetProfile).count()
+    template_dict['response_count'] = \
+      Response.objects.filter(owner=targetProfile).count()
+    template_dict['announcement_count'] = \
+      Announcement.objects.filter(incumbent=targetProfile).count()
+    template_dict['event_count'] = \
+      Event.objects.filter(owner=targetProfile).count()
+
+    return render_to_response(
+        'custom_modify_user.html',
+        template_dict,
+        context_instance=RequestContext(request),
+        )
 
 @admin_required
 def custom_add_user_view(request):
@@ -669,7 +722,11 @@ def reset_pw_confirm_view(request, uidb64=None, token=None):
 
 @admin_required
 def recount_view(request):
-    ''' Recount number_of_messages for all threads and number_of_responses for all requests. '''
+    """
+    Recount number_of_messages for all threads and number_of_responses for all requests.
+    Also set the change_date for every thread to the post_date of the latest message
+    associated with that thread.
+    """
     requests_changed = 0
     for req in Request.objects.all():
         recount = Response.objects.filter(request=req).count()
@@ -684,14 +741,22 @@ def recount_view(request):
             thread.number_of_messages = recount
             thread.save()
             threads_changed += 1
+    dates_changed = 0
+    for thread in Thread.objects.all():
+        if thread.change_date != thread.message_set.latest('post_date').post_date:
+            thread.change_date = thread.message_set.latest('post_date').post_date
+            thread.save()
+            dates_changed += 1
     messages.add_message(request, messages.SUCCESS, MESSAGES['RECOUNTED'].format(
         requests_changed=requests_changed,
         request_count=Request.objects.all().count(),
         threads_changed=threads_changed,
-        thread_count=Thread.objects.all().count()),
-        )
+        thread_count=Thread.objects.all().count(),
+        dates_changed=dates_changed,
+        ))
     return HttpResponseRedirect(reverse('utilities'))
 
+@profile_required
 def archives_view(request):
     """ View of the archives page. """
     resident_count = UserProfile.objects.filter(status=UserProfile.RESIDENT).count()
@@ -706,6 +771,10 @@ def archives_view(request):
         'thread_count': Thread.objects.all().count(),
         'message_count': Message.objects.all().count(),
         'request_count': Request.objects.all().count(),
+        'expired_count': Request.objects.filter(status=Request.EXPIRED).count(),
+        'filled_count': Request.objects.filter(status=Request.FILLED).count(),
+        'closed_count': Request.objects.filter(status=Request.CLOSED).count(),
+        'open_count': Request.objects.filter(status=Request.OPEN).count(),
         'response_count': Response.objects.all().count(),
         'announcement_count': Announcement.objects.all().count(),
         'event_count': Event.objects.all().count(),
@@ -714,3 +783,162 @@ def archives_view(request):
         'shift_log_entry_count': ShiftLogEntry.objects.all().count(),
         'workshift_instance_count': WorkshiftInstance.objects.all().count(),
         }, context_instance=RequestContext(request))
+
+
+def get_updates_view(request):
+    """Return a user's updates. AJAX."""
+    if not request.is_ajax():
+        raise Http404
+
+    if not request.user.is_authenticated():
+        return HttpResponse(json.dumps(dict()),
+                            content_type="application/json")
+
+    try:
+        user_profile = UserProfile.objects.get(user=request.user)
+    except UserProfile.DoesNotExist:
+        return HttpResponse(json.dumps(dict()),
+                            content_type="application/json")
+
+    response = dict()
+    if request.user.is_superuser:
+        num_of_profile_requests = ProfileRequest.objects.all().count()
+
+        if num_of_profile_requests == 0:
+            response['profile_requests_link'] = '''
+                <span class="glyphicon glyphicon-inbox"></span>
+                Profile Requests
+            '''
+
+        else:
+            response['profile_requests_link'] = """
+                <span title="{req_num} open profile request{mult}"
+                    class="badge pull-right">{req_num}</span>
+                <span class="glyphicon glyphicon-inbox"></span>
+                Profile Requests
+            """.format(
+                req_num=num_of_profile_requests,
+                mult='s' if num_of_profile_requests > 1 else '',
+            )
+
+    notification_count = request.user.notifications.unread().count()
+
+    if notification_count == 0:
+        response['notifications_link'] = '''
+            <span class="glyphicon glyphicon-bell"></span>
+            Notifications
+        '''
+
+        response['profile_dropdown_link'] = '''
+            <span><span class="pull-right"><b class="caret"></b></span>
+            <span class="glyphicon glyphicon-user"></span>
+            {first_name}&nbsp;</span>
+        '''.format(first_name=request.user.first_name)
+
+    else:
+        response['notifications_link'] = """
+            <span title="{n_num} unread notification{mult}"
+                class="badge pull-right">{n_num}</span>
+            <span class="glyphicon glyphicon-bell"></span>
+            Notifications
+        """.format(
+            n_num=notification_count,
+            mult='s' if notification_count > 1 else '',
+        )
+
+        response['profile_dropdown_link'] = '''
+            <span><span class="pull-right"><b class="caret"></b></span><span
+            title="You have {n_num} unread notification{mult}."
+            class="badge pull-right">{n_num}</span>
+            <span class="glyphicon glyphicon-user"></span>
+            {first_name}&nbsp;</span>
+        '''.format(
+            n_num=notification_count,
+            mult='s' if notification_count > 1 else '',
+            first_name=request.user.first_name,
+        )
+
+    req_dict = dict()
+    for req_type in RequestType.objects.filter(enabled=True):
+        open_reqs = Request.objects.filter(request_type=req_type,
+                                          status=Request.OPEN)
+        if not req_type.managers.filter(incumbent__user=request.user):
+            open_reqs = open_reqs.exclude(
+                ~Q(owner__user=request.user), private=True,
+                )
+        num_open = open_reqs.count()
+        if num_open == 0:
+            req_dict['{rtype}_requests_link'.format(rtype=req_type.url_name)] \
+                = """
+                    <span class="glyphicon glyphicon-{icon}"></span>
+                    {name}
+                """.format(
+                    icon=req_type.glyphicon if req_type.glyphicon else 'inbox',
+                    name=req_type.name
+                )
+
+        else:
+            req_dict['{rtype}_requests_link'.format(rtype=req_type.url_name)] \
+                = """
+                    <span title="{num} open request{mult}"
+                        class="badge pull-right">{num}</span>
+                    <span class="glyphicon glyphicon-{icon}"></span>
+                    {name}
+                """.format(
+                    num=num_open,
+                    mult='s' if num_open > 1 else '',
+                    icon=req_type.glyphicon if req_type.glyphicon else 'inbox',
+                    name=req_type.name,
+                )
+
+    if req_dict.keys():
+        response['requests_dict'] = req_dict
+
+    request_pk_list = request.GET.get('request_pk_list', False)
+    if request_pk_list:
+        request_pk_list = request_pk_list.split(',')
+        for request_pk in request_pk_list:
+            try:
+                req = Request.objects.get(pk=request_pk)
+            except Request.DoesNotExist:
+                continue
+
+            response['vote_count_{pk}'.format(pk=req.pk)] = req.upvotes.all().count()
+
+            list_string = 'vote_list_{pk}'.format(pk=request_pk)
+            vote_string = 'in_votes_{pk}'.format(pk=request_pk)
+            count_string = 'vote_count_{pk}'.format(pk=request_pk)
+            response[list_string], response[vote_string], \
+                response[count_string] = build_ajax_votes(
+                    req,
+                    user_profile
+                )
+
+    event_pk_list = request.GET.get('event_pk_list', False)
+    if event_pk_list:
+        event_pk_list = event_pk_list.split(',')
+        for event_pk in event_pk_list:
+            try:
+                event = Event.objects.get(pk=event_pk)
+            except Event.DoesNotExist:
+                continue
+
+            link_string = 'rsvp_link_{pk}'.format(pk=event.pk)
+            list_string = 'rsvp_list_{pk}'.format(pk=event.pk)
+            response[link_string], response[list_string] = build_ajax_rsvps(
+                event,
+                user_profile
+            )
+
+    thread_pk = request.GET.get('thread_pk', False)
+    if thread_pk:
+        try:
+            thread = Thread.objects.get(pk=thread_pk)
+        except Thread.DoesNotExist:
+            pass
+        else:
+            response['following'] = user_profile in thread.followers.all()
+            response['num_of_followers'] = thread.followers.all().count()
+
+    return HttpResponse(json.dumps(response),
+                        content_type="application/json")
